@@ -6,25 +6,44 @@ django-helpdesk - A Django powered ticket tracker for small enterprise.
 lib.py - Common functions (eg multipart e-mail)
 """
 
-chart_colours = ('80C65A', '990066', 'FF9900', '3399CC', 'BBCCED', '3399CC', 'FFCC33')
+import logging
+import mimetypes
+import os
+from smtplib import SMTPException
 
 try:
+    # Python 2 support
     from base64 import urlsafe_b64encode as b64encode
 except ImportError:
-    from base64 import encodestring as b64encode
+    # Python 3 support
+    from base64 import encodebytes as b64encode
 try:
+    # Python 2 support
     from base64 import urlsafe_b64decode as b64decode
 except ImportError:
-    from base64 import decodestring as b64decode
+    # Python 3 support
+    from base64 import decodebytes as b64decode
+
+from django.conf import settings
+from django.db.models import Q
+from django.utils import six
+from django.utils.encoding import smart_text
+from django.utils.safestring import mark_safe
+
+from helpdesk.models import Attachment, EmailTemplate
 
-import logging
 logger = logging.getLogger('helpdesk')
 
-from django.utils.encoding import smart_str
 
-def send_templated_mail(template_name, email_context, recipients, sender=None, bcc=None, fail_silently=False, files=None):
+def send_templated_mail(template_name,
+                        context,
+                        recipients,
+                        sender=None,
+                        bcc=None,
+                        fail_silently=False,
+                        files=None):
     """
-    send_templated_mail() is a warpper around Django's e-mail routines that
+    send_templated_mail() is a wrapper around Django's e-mail routines that
     allows us to easily send multipart (text/plain & text/html) e-mails using
     templates that are stored in the database. This lets the admin provide
     both a text and a HTML template for each message.
@@ -32,7 +51,7 @@ def send_templated_mail(template_name, email_context, recipients, sender=None, b
     template_name is the slug of the template to use for this message (see
         models.EmailTemplate)
 
-    email_context is a dictionary to be used when rendering the template
+    context is a dictionary to be used when rendering the template
 
     recipients can be either a string, eg 'a@b.com', or a list of strings.
 
@@ -45,91 +64,87 @@ def send_templated_mail(template_name, email_context, recipients, sender=None, b
     fail_silently is passed to Django's mail routine. Set to 'True' to ignore
         any errors at send time.
 
-    files can be a list of tuple. Each tuple should be a filename to attach, 
+    files can be a list of tuples. Each tuple should be a filename to attach,
         along with the File objects to be read. files can be blank.
 
     """
-    from django.conf import settings
     from django.core.mail import EmailMultiAlternatives
-    from django.template import loader, Context
+    from django.template import engines
+    from_string = engines['django'].from_string
 
     from helpdesk.models import EmailTemplate
-    from helpdesk.settings import HELPDESK_EMAIL_SUBJECT_TEMPLATE
-    import os
+    from helpdesk.settings import HELPDESK_EMAIL_SUBJECT_TEMPLATE, \
+        HELPDESK_EMAIL_FALLBACK_LOCALE
 
-    context = Context(email_context)
+    locale = context['queue'].get('locale') or HELPDESK_EMAIL_FALLBACK_LOCALE
 
-    if hasattr(context['queue'], 'locale'):
-        locale = getattr(context['queue'], 'locale', '')
-    else:
-        locale = context['queue'].get('locale', 'en')
-    if not locale:
-        locale = 'en'
-
-    t = None
     try:
         t = EmailTemplate.objects.get(template_name__iexact=template_name, locale=locale)
     except EmailTemplate.DoesNotExist:
-        pass
-
-    if not t:
         try:
             t = EmailTemplate.objects.get(template_name__iexact=template_name, locale__isnull=True)
         except EmailTemplate.DoesNotExist:
-            logger.warning('template "%s" does not exist, no mail sent' %
-			   template_name)
-            return # just ignore if template doesn't exist
+            logger.warning('template "%s" does not exist, no mail sent', template_name)
+            return  # just ignore if template doesn't exist
 
-    if not sender:
-        sender = settings.DEFAULT_FROM_EMAIL
+    subject_part = from_string(
+        HELPDESK_EMAIL_SUBJECT_TEMPLATE % {
+            "subject": t.subject
+        }).render(context).replace('\n', '').replace('\r', '')
 
     footer_file = os.path.join('helpdesk', locale, 'email_text_footer.txt')
 
-    text_part = loader.get_template_from_string(
+    text_part = from_string(
         "%s{%% include '%s' %%}" % (t.plain_text, footer_file)
-        ).render(context)
+    ).render(context)
 
     email_html_base_file = os.path.join('helpdesk', locale, 'email_html_base.html')
+    # keep new lines in html emails
+    if 'comment' in context:
+        context['comment'] = mark_safe(context['comment'].replace('\r\n', '<br>'))
 
+    html_part = from_string(
+        "{%% extends '%s' %%}{%% block title %%}"
+        "%s"
+        "{%% endblock %%}{%% block content %%}%s{%% endblock %%}" %
+        (email_html_base_file, t.heading, t.html)
+    ).render(context)
 
-    ''' keep new lines in html emails '''
-    from django.utils.safestring import mark_safe
-
-    if context.has_key('comment'):
-        html_txt = context['comment']
-        html_txt = html_txt.replace('\r\n', '<br>')
-        context['comment'] = mark_safe(html_txt)
-
-    html_part = loader.get_template_from_string(
-        "{%% extends '%s' %%}{%% block title %%}%s{%% endblock %%}{%% block content %%}%s{%% endblock %%}" % (email_html_base_file, t.heading, t.html)
-        ).render(context)
-
-    subject_part = loader.get_template_from_string(
-        HELPDESK_EMAIL_SUBJECT_TEMPLATE % {
-            "subject": t.subject,
-        }).render(context)
-
-    if isinstance(recipients,(str,unicode)):
+    if isinstance(recipients, str):
         if recipients.find(','):
             recipients = recipients.split(',')
     elif type(recipients) != list:
-        recipients = [recipients,]
+        recipients = [recipients]
 
-    msg = EmailMultiAlternatives(   subject_part.replace('\n', '').replace('\r', ''),
-                                    text_part,
-                                    sender,
-                                    recipients,
-                                    bcc=bcc)
+    msg = EmailMultiAlternatives(subject_part, text_part,
+                                 sender or settings.DEFAULT_FROM_EMAIL,
+                                 recipients, bcc=bcc)
     msg.attach_alternative(html_part, "text/html")
 
     if files:
-        for attachment in files:
-            file_to_attach = attachment[1]
-            file_to_attach.open()
-            msg.attach(filename=attachment[0], content=file_to_attach.read())
-            file_to_attach.close()
+        for filename, filefield in files:
+            mime = mimetypes.guess_type(filename)
+            if mime[0] is not None and mime[0] == "text/plain":
+                with open(filefield.path, 'r') as attachedfile:
+                    content = attachedfile.read()
+                    msg.attach(filename, content)
+            else:
+                if six.PY3:
+                    msg.attach_file(filefield.path)
+                else:
+                    with open(filefield.path, 'rb') as attachedfile:
+                        content = attachedfile.read()
+                        msg.attach(filename, content)
 
-    return msg.send(fail_silently)
+    logger.debug('Sending email to: {!r}'.format(recipients))
+
+    try:
+        return msg.send()
+    except SMTPException as e:
+        logger.exception('SMTPException raised while sending email to {}'.format(recipients))
+        if not fail_silently:
+            raise e
+        return 0
 
 
 def query_to_dict(results, descriptions):
@@ -163,17 +178,27 @@ def apply_query(queryset, params):
     params is a dictionary that contains the following:
         filtering: A dict of Django ORM filters, eg:
             {'user__id__in': [1, 3, 103], 'title__contains': 'foo'}
-        other_filter: Another filter of some type, most likely a
-            set of Q() objects.
+
+        search_string: A freetext search string
+
         sorting: The name of the column to sort by
     """
     for key in params['filtering'].keys():
         filter = {key: params['filtering'][key]}
         queryset = queryset.filter(**filter)
 
-    if params.get('other_filter', None):
-        # eg a Q() set
-        queryset = queryset.filter(params['other_filter'])
+    search = params.get('search_string', None)
+    if search:
+        qset = (
+            Q(title__icontains=search) |
+            Q(description__icontains=search) |
+            Q(resolution__icontains=search) |
+            Q(submitter_email__icontains=search) |
+            Q(ticketcustomfieldvalue__value__icontains=search)
+        )
+
+        # Distinct works, when there are multiple custom fields
+        queryset = queryset.filter(qset).distinct()
 
     sorting = params.get('sorting', None)
     if sorting:
@@ -185,6 +210,38 @@ def apply_query(queryset, params):
     return queryset
 
 
+def ticket_template_context(ticket):
+    context = {}
+
+    for field in ('title', 'created', 'modified', 'submitter_email',
+                  'status', 'get_status_display', 'on_hold', 'description',
+                  'resolution', 'priority', 'get_priority_display',
+                  'last_escalation', 'ticket', 'ticket_for_url',
+                  'get_status', 'ticket_url', 'staff_url', '_get_assigned_to'
+                  ):
+        attr = getattr(ticket, field, None)
+        if callable(attr):
+            context[field] = '%s' % attr()
+        else:
+            context[field] = attr
+    context['assigned_to'] = context['_get_assigned_to']
+
+    return context
+
+
+def queue_template_context(queue):
+    context = {}
+
+    for field in ('title', 'slug', 'email_address', 'from_address', 'locale'):
+        attr = getattr(queue, field, None)
+        if callable(attr):
+            context[field] = attr()
+        else:
+            context[field] = attr
+
+    return context
+
+
 def safe_template_context(ticket):
     """
     Return a dictionary that can be used as a template context to render
@@ -201,32 +258,10 @@ def safe_template_context(ticket):
     """
 
     context = {
-        'queue': {},
-        'ticket': {},
-        }
-    queue = ticket.queue
-
-    for field in (  'title', 'slug', 'email_address', 'from_address', 'locale'):
-        attr = getattr(queue, field, None)
-        if callable(attr):
-            context['queue'][field] = attr()
-        else:
-            context['queue'][field] = attr
-
-    for field in (  'title', 'created', 'modified', 'submitter_email',
-                    'status', 'get_status_display', 'on_hold', 'description',
-                    'resolution', 'priority', 'get_priority_display',
-                    'last_escalation', 'ticket', 'ticket_for_url',
-                    'get_status', 'ticket_url', 'staff_url', '_get_assigned_to'
-                 ):
-        attr = getattr(ticket, field, None)
-        if callable(attr):
-            context['ticket'][field] = '%s' % attr()
-        else:
-            context['ticket'][field] = attr
-
+        'queue': queue_template_context(ticket.queue),
+        'ticket': ticket_template_context(ticket),
+    }
     context['ticket']['queue'] = context['queue']
-    context['ticket']['assigned_to'] = context['ticket']['_get_assigned_to']
 
     return context
 
@@ -238,14 +273,14 @@ def text_is_spam(text, request):
     # False if it is not spam. If it cannot be checked for some reason, we
     # assume it isn't spam.
     from django.contrib.sites.models import Site
-    from django.conf import settings
+    from django.core.exceptions import ImproperlyConfigured
     try:
         from helpdesk.akismet import Akismet
-    except:
+    except ImportError:
         return False
     try:
         site = Site.objects.get_current()
-    except:
+    except ImproperlyConfigured:
         site = Site(domain='configure-django-sites.com')
 
     ak = Akismet(
@@ -254,10 +289,10 @@ def text_is_spam(text, request):
     )
 
     if hasattr(settings, 'TYPEPAD_ANTISPAM_API_KEY'):
-        ak.setAPIKey(key = settings.TYPEPAD_ANTISPAM_API_KEY)
+        ak.setAPIKey(key=settings.TYPEPAD_ANTISPAM_API_KEY)
         ak.baseurl = 'api.antispam.typepad.com/1.1/'
     elif hasattr(settings, 'AKISMET_API_KEY'):
-        ak.setAPIKey(key = settings.AKISMET_API_KEY)
+        ak.setAPIKey(key=settings.AKISMET_API_KEY)
     else:
         return False
 
@@ -270,6 +305,32 @@ def text_is_spam(text, request):
             'comment_author': '',
         }
 
-        return ak.comment_check(smart_str(text), data=ak_data)
+        return ak.comment_check(smart_text(text), data=ak_data)
 
     return False
+
+
+def process_attachments(followup, attached_files):
+    max_email_attachment_size = getattr(settings, 'MAX_EMAIL_ATTACHMENT_SIZE', 512000)
+    attachments = []
+
+    for attached in attached_files:
+        if attached.size:
+            filename = smart_text(attached.name)
+            att = Attachment(
+                followup=followup,
+                file=attached,
+                filename=filename,
+                mime_type=attached.content_type or
+                mimetypes.guess_type(filename, strict=False)[0] or
+                'application/octet-stream',
+                size=attached.size,
+            )
+            att.save()
+
+            if attached.size < max_email_attachment_size:
+                # Only files smaller than 512kb (or as defined in
+                # settings.MAX_EMAIL_ATTACHMENT_SIZE) are sent via email.
+                attachments.append([filename, att.file])
+
+    return attachments
